import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
/**
 * Clase Model para Cliente a ser usada en JTable
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class ClienteModel extends AbstractTableModel implements KeyListener {
    private java.util.List<Cliente> lst = new ArrayList<Cliente>();
    private JTable tbl = null;
    /**
     * Constructor for objects of class ClienteModel
     */
    public ClienteModel() {
        // cargo dos clientes de prueba
        lst.add(new Cliente(100,"Cliente 100",100.25));
        lst.add(new Cliente(101,"Cliente 101",101.25));
    }
    // CUSTOM
    public void setJTable(JTable t) {
        tbl = t;
    }
    // METODOS QUE PODEMOS SOBRE-ESCRIBIR AbstractTableModel
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
            case 0: // idc
                return Integer.class;
            case 1: // nombre
                return String.class;
            case 2:
                return Double.class;
        }
        return  Object.class;
    }
    public String getColumnName(int column) {
        switch(column) {
            case 0: // idc
                return "Codigo";
            case 1: // nombre
                return "Descripcion";
            case 2:
                return "Saldo";
        }
        return "";
    }
    public boolean isCellEditable(int rowIndex,int columnIndex) {
        if ( columnIndex == 0 ) return false; // no permito cambios en la columna IDC
        return true;
    }
    public void setValueAt(Object aValue,int rowIndex,int columnIndex) {
        Cliente c = lst.get(rowIndex);
        switch(columnIndex) {
            case 1:
                if ( ((String) aValue).length() == 0 ) 
                    JOptionPane.showMessageDialog(null,"Debe Indicar Descripcion","Modifico Cliente", 
                        JOptionPane.ERROR_MESSAGE);
                else c.setDescr((String) aValue);
                break;
            case 2:
                c.setSaldo((Double) aValue);
                break;
        }
    }
    // METODOS ABSTRACTOS OBLIGATORIOS QUE DEBEMOS IMPLEMENTAR
    public int getRowCount() {
        return lst.size(); // la cantidad de elementos que tiene lst
    }
    public int getColumnCount() {
        return 3; // porque cliente tiene 3 atributos: idc,nombre,saldo
    }
    public Object getValueAt(int row, int column) {
        Cliente c = lst.get(row);
        switch(column) {
            case 0: // idc
                return c.getIdc();
            case 1: // nombre
                return c.getDescr();
            case 2:
                return c.getSaldo();
        }
        return  null;
    }
    // METODOS DE LA INTERFAZ KeyListener
    public void keyTyped(KeyEvent e) {
    }
    public void keyPressed(KeyEvent e) {
        if ( e.getKeyCode() == KeyEvent.VK_INSERT ) {
            System.out.println("presiono Insert!");
            String scodigo = null, sdescr = null, ssaldo = null;
            boolean ok=false;
            do {
                try {
                    scodigo = JOptionPane.showInputDialog(null,"Codigo:","Nuevo Cliente",JOptionPane.INFORMATION_MESSAGE);
                    int ncodigo = Integer.parseInt(scodigo);
                    // validacion adicional el codigo no debe estar repetido
                    boolean encontre = false;
                    for(Cliente c : lst) {
                        if ( c.getIdc() == ncodigo ) {
                            encontre = true;
                            break;
                        }
                    }
                    if ( !encontre ) ok=true;
                    else JOptionPane.showMessageDialog(null,"El Codigo Cliente " + scodigo + " ya existe!");
                } catch(NumberFormatException nfe) {
                }
            } while(!ok);
            do {
                sdescr  = JOptionPane.showInputDialog(null,"Descripcion:","Nuevo Cliente",JOptionPane.INFORMATION_MESSAGE);
            } while(sdescr.length() == 0);
            ok=false;
            do {
                try {
                    ssaldo  = JOptionPane.showInputDialog(null,"Saldo Inicial:","Nuevo Cliente",JOptionPane.INFORMATION_MESSAGE);
                    Double.parseDouble(ssaldo);
                    ok=true;
                } catch(NumberFormatException nfe) {
                }
            } while(!ok);
            Cliente c = new Cliente(new Integer(scodigo),sdescr,new Double(ssaldo));
            // agrego nuevo cliente a la lista
            lst.add(c);
            fireTableDataChanged(); // informo a la JTable que he cambiado los datos!
        }
        if ( e.getKeyCode() == KeyEvent.VK_DELETE ) {
            //System.out.println("presiono Delete!");
            if ( tbl != null ) {
                if ( tbl.getSelectedRow() != -1 ) {
                    // pedir confirmacion!
                    if ( JOptionPane.showOptionDialog(null,"¿Borro al Cliente Seleccionado?",
                        "Borro Cliente", JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE,
                            null,new String[] { "Si", "No" } , "Si") == 0 ) {
                        lst.remove(tbl.getSelectedRow());
                        fireTableDataChanged(); // informo a la JTable que he cambiado los datos!
                    }
                } else System.out.println("Debe Seleccionar Cliente a Borrar");
            } else System.out.println("Debe Indicar la JTable asociada a ClienteModel");
        }
    }
    public void keyReleased(KeyEvent e) {
    }
    
}
